@@ -52,8 +52,8 @@ build_time_vars = {'ABIFLAGS': '',
  'CONFINCLUDEDIR': '/usr/local/include',
  'CONFINCLUDEPY': '/usr/local/include/python3.10',
  'COREPYTHONPATH': '',
- 'COVERAGE_INFO': '/usr/obj/ports/Python-3.10.10/Python-3.10.10/coverage.info',
- 'COVERAGE_REPORT': '/usr/obj/ports/Python-3.10.10/Python-3.10.10/lcov-report',
+ 'COVERAGE_INFO': '/build/tmp/pobj/Python-3.10.11/Python-3.10.11/coverage.info',
+ 'COVERAGE_REPORT': '/build/tmp/pobj/Python-3.10.11/Python-3.10.11/lcov-report',
  'COVERAGE_REPORT_OPTIONS': '--no-branch-coverage --title "CPython lcov '
                             'report"',
  'CPPFLAGS': '-I. -I./Include -I/usr/local/include',
@@ -533,11 +533,11 @@ build_time_vars = {'ABIFLAGS': '',
                    '/usr/local/include/python3.10',
  'INCLUDEDIR': '/usr/local/include',
  'INCLUDEPY': '/usr/local/include/python3.10',
- 'INSTALL': '/usr/obj/ports/Python-3.10.10/bin/install -c',
- 'INSTALL_DATA': '/usr/obj/ports/Python-3.10.10/bin/install -c -m 644',
- 'INSTALL_PROGRAM': '/usr/obj/ports/Python-3.10.10/bin/install -c -s -m 755',
- 'INSTALL_SCRIPT': '/usr/obj/ports/Python-3.10.10/bin/install -c -m 755',
- 'INSTALL_SHARED': '/usr/obj/ports/Python-3.10.10/bin/install -c -m 755',
+ 'INSTALL': '/build/tmp/pobj//Python-3.10.11/bin/install -c',
+ 'INSTALL_DATA': '/build/tmp/pobj//Python-3.10.11/bin/install -c -m 644',
+ 'INSTALL_PROGRAM': '/build/tmp/pobj//Python-3.10.11/bin/install -c -s -m 755',
+ 'INSTALL_SCRIPT': '/build/tmp/pobj//Python-3.10.11/bin/install -c -m 755',
+ 'INSTALL_SHARED': '/build/tmp/pobj//Python-3.10.11/bin/install -c -m 755',
  'INSTSONAME': 'libpython3.10.so.0.0',
  'IO_H': 'Modules/_io/_iomodule.h',
  'IO_OBJS': '\\',
@@ -670,13 +670,13 @@ build_time_vars = {'ABIFLAGS': '',
                    '-I/usr/local/include -I./Include/internal -I. -I./Include '
                    '-I/usr/local/include -fPIC -DPy_BUILD_CORE',
  'PY_CORE_LDFLAGS': '-L/usr/local/lib/ '
-                    '-L/usr/obj/ports/Python-3.10.10/Python-3.10.10 '
+                    '-L/build/tmp/pobj//Python-3.10.11/Python-3.10.11 '
                     '-L/usr/local/lib/',
  'PY_CPPFLAGS': '-I. -I./Include -I/usr/local/include',
  'PY_ENABLE_SHARED': 1,
  'PY_FORMAT_SIZE_T': '"z"',
  'PY_LDFLAGS': '-L/usr/local/lib/',
- 'PY_LDFLAGS_NODIST': '-L/usr/obj/ports/Python-3.10.10/Python-3.10.10 '
+ 'PY_LDFLAGS_NODIST': '-L/build/tmp/pobj//Python-3.10.11/Python-3.10.11 '
                       '-L/usr/local/lib/',
  'PY_SSL_DEFAULT_CIPHERS': 1,
  'PY_SSL_DEFAULT_CIPHER_STRING': 0,
@@ -696,7 +696,7 @@ build_time_vars = {'ABIFLAGS': '',
  'READELF': 'readelf',
  'RESSRCDIR': 'Mac/Resources/framework',
  'RETSIGTYPE': 'void',
- 'RUNSHARED': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.10.10/Python-3.10.10',
+ 'RUNSHARED': 'LD_LIBRARY_PATH=/build/tmp/pobj/Python-3.10.11/Python-3.10.11',
  'SCRIPTDIR': '/usr/local/lib',
  'SETPGRP_HAVE_ARG': 1,
  'SHELL': '/bin/sh',
@@ -739,10 +739,10 @@ build_time_vars = {'ABIFLAGS': '',
  'TCLTK_LIBS': '-L/usr/X11R6/lib -ltcl86 -ltk86',
  'TESTOPTS': '',
  'TESTPATH': '',
- 'TESTPYTHON': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.10.10/Python-3.10.10 '
+ 'TESTPYTHON': 'LD_LIBRARY_PATH=/build/tmp/pobj/Python-3.10.11/Python-3.10.11 '
                './python',
  'TESTPYTHONOPTS': '',
- 'TESTRUNNER': 'LD_LIBRARY_PATH=/usr/obj/ports/Python-3.10.10/Python-3.10.10 '
+ 'TESTRUNNER': 'LD_LIBRARY_PATH=/build/tmp/pobj/Python-3.10.11/Python-3.10.11 '
                './python ./Tools/scripts/run_tests.py',
  'TESTSUBDIRS': 'ctypes/test \\',
  'TESTTIMEOUT': 1200,
@@ -770,8 +770,8 @@ build_time_vars = {'ABIFLAGS': '',
  'WITH_VALGRIND': 0,
  'X87_DOUBLE_ROUNDING': 0,
  'XMLLIBSUBDIRS': 'xml xml/dom xml/etree xml/parsers xml/sax',
- 'abs_builddir': '/usr/obj/ports/Python-3.10.10/Python-3.10.10',
- 'abs_srcdir': '/usr/obj/ports/Python-3.10.10/Python-3.10.10',
+ 'abs_builddir': '/build/tmp/pobj/Python-3.10.11/Python-3.10.11',
+ 'abs_srcdir': '/build/tmp/pobj/Python-3.10.11/Python-3.10.11',
  'datarootdir': '/usr/local/share',
  'exec_prefix': '/usr/local',
  'prefix': '/usr/local',
